import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Filters } from 'nmr-processing';
import { getSlice } from '../../../../data/data2d/Spectrum2D';
import { useChartData } from '../../../context/ChartContext';
import { useActiveSpectrum } from '../../../hooks/useActiveSpectrum';
import useSpectrum from '../../../hooks/useSpectrum';
import { PathBuilder } from '../../../utility/PathBuilder';
import { get2DXScale, get2DYScale, getSliceYScale, useScale2DX, useScale2DY, } from '../../utilities/scale';
import { useActivePhaseTraces } from './useActivePhaseTraces';
function usePath(x, y, direction) {
    const { width, margin, height, xDomain, yDomain, mode } = useChartData();
    const { scaleRatio } = useActivePhaseTraces();
    const spectrum = useSpectrum();
    if (!spectrum)
        return '';
    if (direction === 'horizontal') {
        const scaleX = get2DXScale({ margin, width, xDomain, mode });
        const scaleY = getSliceYScale(spectrum.data, height, 'RTL', {
            margin: margin.top + margin.bottom,
            scaleRatio,
        });
        const pathBuilder = new PathBuilder();
        pathBuilder.moveTo(scaleX(x[0]), scaleY(y[0]));
        for (let i = 1; i < x.length; i++) {
            pathBuilder.lineTo(scaleX(x[i]), scaleY(y[i]));
        }
        return pathBuilder.toString();
    }
    const scaleX = get2DYScale({ margin, height, yDomain });
    const scaleY = getSliceYScale(spectrum.data, width, 'RTL', {
        margin: margin.left + margin.right,
        scaleRatio,
    });
    const pathBuilder = new PathBuilder();
    pathBuilder.moveTo(scaleY(y.at(-1)), scaleX(x.at(-1)));
    for (let i = x.length - 2; i >= 0; i--) {
        pathBuilder.lineTo(scaleY(y[i]), scaleX(x[i]));
    }
    return pathBuilder.toString();
}
export function SpectrumPhaseTrace(props) {
    const { positionUnit, position, children, ...otherProps } = props;
    const { tempData, width, margin, height } = useChartData();
    const { activeTraceDirection: direction } = useActivePhaseTraces();
    const scale2dX = useScale2DX();
    const scale2dY = useScale2DY();
    const activeSpectrum = useActiveSpectrum();
    if (!activeSpectrum?.id) {
        return null;
    }
    const spectrumBeforePhasing = tempData[activeSpectrum.index];
    let positionInPixel;
    let positionInPPM;
    if (positionUnit === 'Pixel') {
        positionInPixel = position;
        positionInPPM = {
            x: scale2dX.invert(position.x),
            y: scale2dY.invert(position.y),
        };
    }
    else {
        positionInPixel = {
            x: scale2dX(position.x),
            y: scale2dY(position.y),
        };
        positionInPPM = position;
    }
    const sliceData = getSlice(spectrumBeforePhasing, positionInPPM, {
        sliceType: 'both',
    });
    const data = sliceData?.[direction]?.data;
    if (!data) {
        return null;
    }
    const innerHeight = height - margin.top - margin.bottom;
    const innerWidth = width - margin.left - margin.right;
    const translateY = direction === 'horizontal' ? positionInPixel.y - innerHeight : 0;
    const translateX = direction === 'vertical' ? positionInPixel.x - innerWidth : 0;
    return (_jsx(InnerSpectrumPhaseTrace, { transform: `translate(${translateX} ${translateY})`, ...otherProps, data: data, children: children }));
}
function InnerSpectrumPhaseTrace(props) {
    const { data: dataBeforePhasing, children, ...othersProps } = props;
    const { color, activeTraceDirection: direction, ph0, ph1, } = useActivePhaseTraces();
    const spectrum = {
        data: dataBeforePhasing,
        info: { isComplex: true, isFid: false },
    };
    Filters.phaseCorrection.apply(spectrum, {
        ph0,
        ph1,
    });
    const { x, re } = spectrum.data;
    const path = usePath(x, re, direction);
    return (_jsxs("g", { ...othersProps, children: [_jsx("path", { className: "line", stroke: color, strokeWidth: "1", fill: "transparent", d: path, pointerEvents: "none" }), children] }));
}
